@@ -19,16 +19,19 @@ except ImportError:
 logdir = os.path.expanduser("~")
 logdir = os.path.join(logdir, ".tech-parser")
 
-def get_similarity(article1, article2):
-	pairs1 = get_pairs(prepare_string(article1['title']))
-	pairs2 = get_pairs(prepare_string(article2['title']))
-	
+r1 = re.compile(r"(?P<g1>\w+)n['\u2019]t", re.UNICODE)
+r2 = re.compile(r"(?P<g1>\w+)['\u2019]s", re.UNICODE)
+r3 = re.compile(r"(?P<g1>\w+)['\u2019]m", re.UNICODE)
+r4 = re.compile(r"(?P<g1>\w+)['\u2019]re", re.UNICODE)
+r5 = re.compile(r"(?P<g1>\w+)['\u2019]ve", re.UNICODE)
+r6 = re.compile(r"(?P<g1>\w+)['\u2019]d", re.UNICODE)
+r7 = re.compile(r"(?P<g1>\w+)['\u2019]ll", re.UNICODE)
+r8 = re.compile(r"gonna", re.UNICODE)
+r9 = re.compile(r"\W", re.UNICODE)
+
+def get_similarity(pairs1, pairs2):
 	len_all_pairs = len(pairs1) + len(pairs2)
-	shrd = []
-	for pair in pairs1:
-		if pair in pairs2:
-			if shrd.count(pair) < min(pairs1.count(pair), pairs2.count(pair)):
-				shrd.append(pair)
+	shrd = pairs1 & pairs2
 	
 	if len_all_pairs == 0:
 		return 0.0
@@ -39,37 +42,45 @@ def find_similiar(articles, db='sqlite'):
 	interesting_articles = get_interesting_articles(db)
 	blacklist = get_blacklist(db)
 	ignored_links = [i['link'] for i in blacklist]
-	similiar_articles = []
+	processed, scores = [], []
 	interesting_links = [i['link']
 		for i in interesting_articles]
 	
+	interesting_pairs = [get_pairs(prepare_string(i['title']))
+		for i in interesting_articles[:75]]
+	ignored_pairs = [get_pairs(prepare_string(i['title']))
+		for i in blacklist[:75]]
+	
+	while len(interesting_pairs) > len(ignored_pairs):
+		ignored_pairs.append(set())
+	while len(ignored_pairs) > len(interesting_pairs):
+		interesting_pairs.append(set())
+	
+	zipped = list(zip(interesting_pairs, ignored_pairs))
+	
 	for article in articles:
 		if article['link'] in interesting_links or \
 		article['link'] in ignored_links:
 			continue
 		
 		score = 0.0
-		for interesting_article in interesting_articles:
-			score += get_similarity(article, interesting_article)
-		for ignored_article in blacklist:
-			score -= get_similarity(article, ignored_article)
 		
-		if [article, score] not in similiar_articles:
-			similiar_articles.append([article, score])
+		pairs1 = get_pairs(prepare_string(article['title']))
+		
+		for (pairs2, pairs3) in zipped:
+			if pairs2:
+				score += get_similarity(pairs1, pairs2)
+			if pairs3:
+				score -= get_similarity(pairs1, pairs3)
+		
+		processed.append(article)
+		scores.append(score)
 	
-	return similiar_articles
+	return [[a, s] for (a, s) in zip(processed, scores)]
 
 def prepare_string(s, exclude=["a", "an", "the", "is", "am",
 		"are", "for", "that", "of", "to", "so", "in", "on"]):
 	s = s.strip().lower()
-	r1 = re.compile(r"(?P<g1>\w+)n['\u2019]t", re.UNICODE)
-	r2 = re.compile(r"(?P<g1>\w+)['\u2019]s", re.UNICODE)
-	r3 = re.compile(r"(?P<g1>\w+)['\u2019]m", re.UNICODE)
-	r4 = re.compile(r"(?P<g1>\w+)['\u2019]re", re.UNICODE)
-	r5 = re.compile(r"(?P<g1>\w+)['\u2019]ve", re.UNICODE)
-	r6 = re.compile(r"(?P<g1>\w+)['\u2019]d", re.UNICODE)
-	r7 = re.compile(r"(?P<g1>\w+)['\u2019]ll", re.UNICODE)
-	r8 = re.compile(r"gonna", re.UNICODE)
 	s = r1.sub("\g<g1> not", s)
 	s = r2.sub("\g<g1>", s)
 	s = r3.sub("\g<g1> am", s)
@@ -78,20 +89,14 @@ def prepare_string(s, exclude=["a", "an", "the", "is", "am",
 	s = r6.sub("\g<g1> would", s)
 	s = r7.sub("\g<g1> will", s)
 	s = r8.sub("going to", s)
-	words = re.split(r"\W", s)
-	for word in exclude:
-		while words.count(word) > 0:
-			words.remove(word)
+	words = r9.split(s)
 	
-	return [word for word in words if len(word)]
+	return [word for word in words if len(word) and word not in exclude]
 
 def get_pairs(words):
-	pairs = []
+	pairs = set()
 	for word in words:
-		if len(word) == 1:
-			pairs.append(word)
-		else:
-			pairs += [word[i:i+2] for i in range(len(word))]
+		pairs.update({word[i:i+2] for i in range(len(word))})
 	return pairs
 
 def get_interesting_articles(db='sqlite'):
@@ -99,26 +104,34 @@ def get_interesting_articles(db='sqlite'):
 	connect, args, kwargs = which_db(db)
 	con = connect(*args, **kwargs)
 	cur = con.cursor()
-	cur.execute('SELECT * FROM interesting_articles;')
+	cur.execute('''SELECT title, link, summary, fromrss, icon, color, source
+		FROM interesting_articles ORDER BY id DESC;''')
 	res = cur.fetchall()
 	con.close()
-	return [{'title': x[1],
-			'link': x[2],
-			'summary': x[3],
-			'source': x[4]} for x in res]
+	return [{'title': x[0],
+			'link': x[1],
+			'summary': x[2],
+			'fromrss': x[3],
+			'icon': x[4],
+			'color': x[5],
+			'source': x[6]} for x in res]
 
 def get_blacklist(db='sqlite'):
 	setup_db(db)
 	connect, args, kwargs = which_db(db)
 	con = connect(*args, **kwargs)
 	cur = con.cursor()
-	cur.execute('SELECT * FROM blacklist;')
+	cur.execute('''SELECT title, link, summary, fromrss, icon, color, source
+		FROM blacklist ORDER BY id DESC;''')
 	res = cur.fetchall()
 	con.close()
-	return [{'title': x[1],
-			'link': x[2],
-			'summary': x[3],
-			'source': x[4]} for x in res]
+	return [{'title': x[0],
+			'link': x[1],
+			'summary': x[2],
+			'fromrss': x[3],
+			'icon': x[4],
+			'color': x[5],
+			'source': x[6]} for x in res]
 
 def add_article(addr, db='sqlite'):
 	f = open(os.path.join(logdir, "articles_dumped"), 'rb')
@@ -148,13 +161,15 @@ def add_to_interesting(article, db='sqlite'):
 	con = connect(*args, **kwargs)
 	cur = con.cursor()
 	cur.execute('SELECT count(link) FROM interesting_articles;')
-	if cur.fetchone()[0] > 75:
+	if cur.fetchone()[0] > 1000:
 		cur.execute("""DELETE FROM interesting_articles
 			WHERE id = (SELECT MIN(id) FROM interesting_articles);""")
 	sqlite_code = """INSERT INTO
-			interesting_articles(title, link, summary, source) VALUES(?, ?, ?, ?);"""
+			interesting_articles(title, link, summary, fromrss, icon, color, source)
+				VALUES(?, ?, ?, ?, ?, ?, ?);"""
 	postgres_code = """INSERT INTO
-			interesting_articles(title, link, summary, source) VALUES(%s, %s, %s, %s);"""
+			interesting_articles(title, link, summary, fromrss, icon, color, source)
+				VALUES(%s, %s, %s, %s, %s, %s, %s);"""
 	
 	if db == 'sqlite':
 		code = sqlite_code
@@ -162,8 +177,15 @@ def add_to_interesting(article, db='sqlite'):
 		code = postgres_code
 	
 	try:
-		cur.execute(code, (article[0]['title'], article[0]['link'],
-			article[0]['summary'], article[0]['source']))
+		title = article[0]['title']
+		link = article[0]['link']
+		summary = article[0]['summary']
+		source = article[0]['source']
+		fromrss = article[0].get('fromrss', 0)
+		icon = article[0].get('icon', '')
+		color = article[0].get('color', '#000')
+		
+		cur.execute(code, (title, link, summary, fromrss, icon, color, source))
 		con.commit()
 	except IntegrityError:
 		pass
@@ -201,15 +223,13 @@ def add_to_blacklist(article, db='sqlite'):
 	
 	con = connect(*args, **kwargs)
 	cur = con.cursor()
-	cur.execute('SELECT count(link) FROM blacklist;')
-	if cur.fetchone()[0] > 75:
-		cur.execute("""DELETE FROM blacklist
-			WHERE id == (SELECT MIN(id) FROM blacklist);""")
 
 	sqlite_code = """INSERT INTO
-			blacklist(title, link, summary, source) VALUES(?, ?, ?, ?);"""
+			blacklist(title, link, summary, fromrss, icon, color, source)
+				VALUES(?, ?, ?, ?, ?, ?, ?);"""
 	postgres_code = """INSERT INTO
-			blacklist(title, link, summary, source) VALUES(%s, %s, %s, %s);"""
+			blacklist(title, link, summary, fromrss, icon, color, source)
+				VALUES(%s, %s, %s, %s, %s, %s, %s);"""
 	
 	if db == 'sqlite':
 		code = sqlite_code
@@ -217,8 +237,14 @@ def add_to_blacklist(article, db='sqlite'):
 		code = postgres_code
 	
 	try:
-		cur.execute(code, (article[0]['title'], article[0]['link'],
-			article[0]['summary'], article[0]['source']))
+		title = article[0]['title']
+		link = article[0]['link']
+		summary = article[0]['summary']
+		source = article[0]['source']
+		fromrss = article[0].get('fromrss', 0)
+		icon = article[0].get('icon', '')
+		color = article[0].get('color', '#000')
+		cur.execute(code, (title, link, summary, fromrss, icon, color, source))
 		con.commit()
 	except IntegrityError:
 		pass
@@ -267,16 +293,20 @@ def setup_db(db='sqlite'):
 	sqlite_code = """CREATE TABLE IF NOT EXISTS interesting_articles
 			(id INTEGER PRIMARY KEY AUTOINCREMENT,
 				title TEXT, link TEXT, summary TEXT, source TEXT,
+				fromrss INTEGER, icon TEXT, color TEXT,
 				UNIQUE(link));
 		CREATE TABLE IF NOT EXISTS blacklist
 			(id INTEGER PRIMARY KEY AUTOINCREMENT,
+				fromrss INTEGER, icon TEXT, color TEXT,
 				title TEXT, link TEXT, summary TEXT, source TEXT,
 				UNIQUE(link))"""
 	postgres_code = """CREATE TABLE IF NOT EXISTS interesting_articles
 			(id SERIAL,	title TEXT, link TEXT, summary TEXT,
+				fromrss INTEGER, icon TEXT, color TEXT,
 				source TEXT, UNIQUE(link));
 		CREATE TABLE IF NOT EXISTS blacklist
 			(id SERIAL, title TEXT, link TEXT, summary TEXT,
+				fromrss INTEGER, icon TEXT, color TEXT,
 				source TEXT, UNIQUE(link))"""
 	if db == 'sqlite':
 		code = sqlite_code
